{% block doc -%}

<!DOCTYPE html>
<html {% block html_attribs %}{% endblock html_attribs %}>

{%- block html %}
  {% set icon_image = 'img/production.png' %}
  {% set title = 'Ensembl Production Services' %}

  <head>
    {%- block head %}
    <title>{{title}}</title>

      {%- block metas %}
      <meta name="viewport" content="width=device-width, initial-scale=1.0">
      {%- endblock metas %}

      {%- block styles %}
      <!-- Bootstrap -->
      {{ bootstrap.load_css(version='4.3.0') }}
      <link rel="stylesheet" href="{{ url_for('static', filename='ensembl.css') }}">
      <link rel="shortcut icon" href="{{ url_for('static', filename=icon_image) }}">
      {%- endblock styles %}
    {%- endblock head %}
  </head>

  <body {% block body_attribs %}{% endblock body_attribs %}>
    {% block body -%}
      {% block heading %}
      <a id="top"></a>

      <div class="container">
        <h2><img src="{{ url_for('static', filename=icon_image) }}" /> {{title}}</h2>
      </div>
      {%- endblock heading %}

      {% block navbar %}
      {%- endblock navbar %}

      {% block content -%}
      {%- endblock content %}

      {% block scripts %}
        {{ bootstrap.load_js(version='4.3.0') }}
      {%- endblock scripts %}
    {%- endblock body %}
  </body>

{%- endblock html %}
</html>

{% endblock doc -%}
