
{% extends 'ensembl/base.html' %}

{% block content %}
  <div class="container">
    {% block detail -%}

    <h3>Submission Type: {{ submission_type }}</h3>
    <div>Job ID: {{ job.id }}</div>
    <div>Status: {{ job.status }}</div>

    {%- endblock detail %}
  </div>
{% endblock content %}
