
{% extends 'ensembl/base.html' %}

{% block content -%}

  <div class="container">
    <ul>
      <li><a href="datacheck/names">Datacheck Catalog</a></li>
      <li><a href="datacheck/submit">Datacheck Submission</a></li>
      <li><a href="gifts/submit">GIFTs Submission</a></li>
      <li>etc...</li>
    </ul>
  </div>

{%- endblock %}
